#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" autosys package """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal/autosys
# https://www.twitter.com/skeptycal

# `AutoSys` is licensed under the `MIT <https://opensource.org/licenses/MIT>`.
# from __future__ import absolute_import, print_function
# from autosys import *

if True:  # import builtins
    import decimal
    import fileinput
    import inspect
    import locale
    import logging
    import math
    import os
    import pathlib
    import sys
    import textwrap
    import time
    import timeit
    import traceback

if True:  # other imports
    from typing import Any, Dict, FrozenSet, List, Sequence, Tuple

if True:  # designed to be run globally from directory in system path
    # the sys.path starts with the directory containing __file__ which we want to remove as
    # that dir might be anywhere and could contain anything; it's not needed for locating
    # autosys modules since those will be in site-packages once autosys is installed
    script_path = os.path.abspath(sys.path[0])
    sys.path = [p for p in sys.path if os.path.abspath(p) != script_path]
    # opposite of:
    # sys.path.insert(0, os.path.abspath("."))  # insert pwd
    # sys.path.insert(0, os.path.abspath('autosys'))

if True:  # package defaults
    # TODO setup a way to automatically track semvers
    __version__: str = "1.1.0"
    version: str = __version__
    __version_info__: Tuple[int] = [int(_) for _ in version.split('.')]

    # set default package name to parent folder name
    name: str = __file__.split("/")[-2]
    __package__: str = name

    __license__: str = "MIT <https://opensource.org/licenses/MIT>"
    __author__: str = "Michael Treanor <https://www.github.com/skeptycal>"

if __name__ == "__main__":  # CLI tests
    # assorted import tests
    import pkgutil
    from autosys.as_system import njoin
    from autosys.as_constants import PY_ENV

    MAIN = PY_ENV.get('MAIN')
    CANARY = PY_ENV.get("CANARY")
    RESET = PY_ENV.get("RESET_FG")

    def v_name(the_var: Any) -> str:
        try:
            result = [
                var_name for var_name, var_val in
                inspect.currentframe().f_back.f_back.f_locals.items()
                if var_val is the_var
            ][0]
            return result
        except IndexError as e:
            return ''

    def vprint(the_var: Any):
        print(f'{MAIN}{v_name(the_var)} => {CANARY}{the_var}{RESET}')

    def cprint(s: str):
        print(f'{MAIN}{s}{RESET}')

    print()
    cprint('dir(pkgutil)')
    print(dir(pkgutil))

    print()
    cprint('builtin modules: ')
    print(njoin(sys.builtin_module_names))
    print()

    # set to None to see all modules importable from sys.path
    # reference: https://chrisyeh96.github.io/2017/08/08/definitive-guide-python-imports.html
    # search_path = None
    # search_path = ['.']
    search_path = ['autosys']
    cprint(f'all importable modules in search_path({search_path}).')
    all_modules = [x[1] for x in pkgutil.iter_modules(path=search_path)]
    print(njoin(all_modules))

    print()
    vprint(license)

    print()
    cprint("Test Values")
    cprint("*" * 40)
    vprint(name)
    vprint(__author__)
    vprint(__license__)
    vprint(__version__)
    vprint(__version_info__)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_constants.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

from autosys import *
# * @AUTOSYS_PARSE: START
##############################################
SET_DEBUG: bool = True
PI: float = math.pi

if True:
    ##############################################
    # custom types
    borderType: FrozenSet[str] = {'single', 'double', 'graphic', 'text'}

    locale.setlocale(locale.LC_ALL, "")

    ##############################################
    # working variables
    i: int = 0
    n: int = 0
    filename: str = ''
    file_list: List[str] = []
    word_list: List[str] = []

    ##############################################
    # Python specific constants
    _PY2 = sys.version_info[0] == 2
    PY_ENV: Dict[str, str] = os.environ
    PY_BASE = os.path.basename(PY_ENV["_"])
    PY_VER: str = ".".join(str(_) for _ in __import__("sys").version_info[:3])
    PY3: bool = (lambda x: [x for x in [False]] and None or x)(True)
    PYTHON_PATH: List[str] = PY_ENV["PYTHONPATH"].split(os.pathsep)


class DEFAULT:
    """ Autosys package defaults """
    # Environment constants
    DICT_DISPLAY_SEPARATOR: str = ": "
    ENCODING: str = locale.getlocale()[1] if locale.getlocale()[1] else "UTF-8"
    LANG: str = locale.getlocale()[0] if locale.getlocale()[0] else "en_US"
    PREFERRED_ENCODING: str = locale.getpreferredencoding()

    CONNECT_ERRORS: Tuple[str] = (ConnectionError, ConnectionAbortedError,
                                  ConnectionResetError, ConnectionRefusedError)


class CLI:
    DISPLAY_WIDTH: int = 80
    FIELD_PADDING: int = 15


class AS_MATH:
    """ Math shortcuts and efficient functions """
    # example lookup tables
    SIN: Dict[float, float] = {
        a / 10: math.sin(math.radians(a / 10))
        for a in range(3600)
    }
    COS: Dict[float, float] = {
        a / 10: math.cos(math.radians(a / 10))
        for a in range(3600)
    }
    TAN: Dict[float, float] = {
        a / 10: math.tan(math.radians(a / 10))
        for a in range(3600)
    }


class C_ERRORS:
    """
    C++ style error messages
        Reference:
        Advanced Bash-Scripting Guide
        <http://tldp.org/LDP/abs/html/exitcodes.html#EXITCODESREF>
        from /usr/include/sysexits.h
        Copyright (c) 1987, 1993
        The Regents of the University of California.  All rights reserved.
    """
    EX_OK = 0,  # successful termination
    EX__BASE = 64,  # base value for error messages
    EX_USAGE = 64,  # command line usage error
    EX_DATAERR = 65,  # data format error
    EX_NOINPUT = 66,  # cannot open input
    EX_NOUSER = 67,  # addressee unknown
    EX_NOHOST = 68,  # host name unknown
    EX_UNAVAILABL = 69,  # service unavailable
    EX_SOFTWARE = 70,  # internal software error
    EX_OSERR = 71,  # system error (e.g., can't fork)
    EX_OSFILE = 72,  # critical OS file missing
    EX_CANTCREAT = 73,  # can't create (user) output file
    EX_IOERR = 74,  # input/output error
    EX_TEMPFAIL = 75,  # temp failure; user is invited to retry
    EX_PROTOCOL = 76,  # remote error in protocol
    EX_NOPERM = 77,  # permission denied
    EX_CONFIG = 78,  # configuration error
    EX__MAX = 78  # maximum listed value


# * @AUTOSYS_PARSE: STOP
if __name__ == "__main__":
    print()
    print(PY_ENV.get("CANARY"))
    print("Test Values")
    print("*" * 40)
    print(PY_ENV.get("RESET_FG"))

    print("name: ", name)
    print("version: ", version)
    print("DEFAULT.ENCODING: ", DEFAULT.ENCODING)
    print("python version 3+: ", PY3)
    print("PY_VER: ", PY_VER)
    print("SIN(0): ", AS_MATH.SIN.get(math.radians(0)))
    print("PYTHON_PATH[0]: ", PYTHON_PATH[0])
    print("PI: ", PI)
    print()
    print("__doc__: ", __doc__)
    print("__file__: ", __file__)
    print("__package__: ", __package__)
    print("__name__: ", __name__)

    # print(AS_MATH.SIN)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_flask.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

from flask import current_app, request

try:
    from ujson import dumps
    JSON_PARSER = 'ujson'
except ImportError as e:
    from json import dumps
    JSON_PARSER = 'json'


def ultrajsonify(*args, **kwargs):
    """This function reimplements ``flask.json.jsonify``
    using ``ujson.dumps`` instead of ``json.dumps``.
    """
    indent = 0
    ensure_ascii = current_app.config.get("JSON_AS_ASCII", True)
    mimetype = current_app.config.get("JSONIFY_MIMETYPE", "application/json")

    if current_app.config["JSONIFY_PRETTYPRINT_REGULAR"] and not request.is_xhr:
        indent = 2

    if args and kwargs:
        raise ValueError(
            "ultrajsonify behavior undefined when passed both args and kwargs"
        )
    elif len(args) == 1:
        data = args[0]
    else:
        data = args or kwargs

    return current_app.response_class(
        dumps(data, indent=indent, ensure_ascii=ensure_ascii), mimetype=mimetype
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_80col.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal


import fileinput
import sys
import pathlib
import textwrap
import locale
from typing import List
from autosys.as_constants import DEFAULT


def create_test_file(filename: str = "testfile.txt", encoding: str = as_constants.DEFAULT_ENCODING) -> str:
    """ Create test file and return resolved Purepath """
    p = pathlib.Path(filename)
    p = p.resolve()
    with open(p, mode="w", encoding=encoding, errors="ignore") as f:
        for n in range(20):
            line_str = "X" * (n + 70) + "\n"
            # print(line_str)
            f.write(line_str)
    return p


def trunc_string_width(s: str) -> str:
    return s


if __name__ == "__main__":

    test_value = """This function wraps the input paragraph such that each line
    in the paragraph is at most width characters long. The wrap method
    returns a list of output lines. The returned list
    is empty if the wrapped
    output has no content."""

    set_width = DEFAULT_CLI_WIDTH
    print("Default Language: ", DEFAULT_LANG)
    print("Default Encoding: ", DEFAULT_ENCODING)
    exit

    if len(sys.argv) < 2:
        print("No arguments given. Creating testfile.txt.")
        file_list.append(create_test_file("testfile.txt"))
        show_test = 1
    else:
        try:
            arg_int = int(sys.argv[1])
        except (IndexError, ValueError):
            # print("Unable to parse code as an integer")
            file_list = sys.argv[1:]
        else:
            # print("first argument is int")
            set_width = arg_int
            file_list = sys.argv[2:]

    with fileinput.input(file_list, inplace=True) as f:
        for line in f:
            # process(line)

            value: str = line.strip()
            # Wrap this text.
            wrapper = textwrap.TextWrapper(width=79)
            word_list = "\n".join(wrapper.wrap(value))

            # Process the line here
            print(word_list)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Minimal Python Module Setup File
Requires Python 3.7 (dataclass support)
"""
from dataclasses import asdict, astuple, dataclass, field
from typing import Any, Dict, List, Tuple

from setuptools import setup

# set default package name to parent folder name


@dataclass(frozen=True)
class SetupConfig:
    # __slots__ = ['name', 'version', 'description', 'url',
    #              'author', 'author_email', 'license', 'packages', 'zip_safe']

    name: str = __file__.split("/")[-2]
    version: str = "0.0.1"
    description: str = "a utility for macOS"
    url: str = "http://github.com/skeptycal/" + name
    author: str = "Michael Treanor"
    author_email: str = "skeptycal@gmail.com"
    license: str = "MIT"
    # packages = []
    zip_safe: bool = False

    def __post_init__(self):
        pass

    def __iter__(self):
        yield from asdict(self)

    def __str__(self):
        return f"{self.name.capitalize()} version {self.version} - {self.description}."


if __name__ == "__main__":

    s = SetupConfig(version="0.1")

    s_iter = iter(s)
    print()
    print(s)
    print()
    print(str(s.__repr__()))
    print()
    print((s.__getattribute__("name")))
    print()
    for k in iter(s):
        print(k, ": ", s.__getattribute__(k))
    print({str(k) + ": " + str(s.__getattribute__(k)) for k in iter(s)})
    # s_dict = dict(iter(s))
    # print(s_dict)
    # print()
    print()



# -*- coding: utf-8 -*-
"""
Tests for colors module
"""

import sys

import pytest
from pytest import raises
from autosys.as_ansi import *

_PY2 = sys.version_info[0] == 2


def test_nocolors():
    """We should have the original text with no parameters"""
    assert color("RED") == "RED"


def test_redcolor():
    """Test if we get a correct red"""
    assert color("RED", fg="red") == "\x1b[31mRED\x1b[0m"


def test_default_color():
    assert color("test", fg="default") == "\x1b[39mtest\x1b[0m"
    assert color("test", bg="default") == "\x1b[49mtest\x1b[0m"
    assert color("test", fg="default",
                 bg="default") == "\x1b[39;49mtest\x1b[0m"


def test_error_on_bad_color_string():
    """Test if we have an exception with a bad string color"""
    with raises(ValueError):
        color("RED", "bozo")


def test_integer_color():
    """Test if we can use a integer as a color"""
    assert color("RED", 1) == "\x1b[38;5;1mRED\x1b[0m"


def test_error_on_bad_color_int():
    """Test if we have an exception with a bad color number"""
    with raises(ValueError):
        color("RED", 911)


def test_tuple_color():
    orange = (255, 165, 0)
    assert color("ORANGE", fg=orange) == "\x1b[38;2;255;165;0mORANGE\x1b[0m"


def test_css_color():
    assert color("ORANGE", fg="orange") == "\x1b[38;2;255;165;0mORANGE\x1b[0m"


def test_background_color():
    """Test the background color with string"""

    assert color("RED", bg="red") == "\x1b[41mRED\x1b[0m"
    assert color("PURPLE", bg="purple") == "\x1b[48;2;128;0;128mPURPLE\x1b[0m"


def test_mixed_color():
    assert (color(
        "PINK/GRAY", fg="pink",
        bg="gray") == "\x1b[38;2;255;192;203;48;2;128;128;128mPINK/GRAY\x1b[0m"
            )
    assert (color(
        "GRAY/PINK", fg="gray",
        bg="pink") == "\x1b[38;2;128;128;128;48;2;255;192;203mGRAY/PINK\x1b[0m"
            )


def test_style_color():
    """Test the style with a string"""
    assert color("BOLD", style="bold") == "\x1b[1mBOLD\x1b[0m"


def test_error_setting_style_color():
    """Test if we have an exception setting the style"""
    with raises(ValueError):
        color("BOLD", style="MAY")


def test_error_on_bad_color_notint():
    """Test if we have an exception with a bad color number"""
    with raises(ValueError):
        color("RED", 911.11)


def test_error_on_bad_style():
    """Test if we have an exception with a bad color number"""
    with raises(ValueError):
        color("RED", bg="cursivas")


def test_remove_color():
    """We can get the original message without the colors"""
    assert strip_color(color("RED", "red")) == "RED"
    assert strip_color(color("text", "red", "green",
                             "bold+underline")) == "text"


def test_remove_color_extra():
    """Test other sequences that color would not add, but that are seen in the wild."""
    assert (strip_color("some\x1b[Kthing") == "something"
            )  # remove EL (erase to end of line)
    # remove truncated style ending
    assert strip_color("some\x1b[mthing") == "something"
    # remove odd color ends
    assert strip_color("some\x1b[49;39mthing") == "something"


def test_ansilen():
    assert ansilen("this") == 4
    assert ansilen("this\x1b[K\x1b[0m") == 4
    assert ansilen(red("this")) == 4
    assert ansilen(color("this", 12, 22, "underline+blink")) == 4


def test_parse_rgb():
    peachpuff = (255, 218, 185)
    assert parse_rgb("peachpuff") == peachpuff
    assert parse_rgb("#ffdab9") == peachpuff
    assert parse_rgb("rgb(255, 218, 185)") == peachpuff
    assert parse_rgb("peachpuff") == peachpuff

    rebeccapurple = (102, 51, 153)
    assert parse_rgb("rebeccapurple") == rebeccapurple
    assert parse_rgb("#639") == rebeccapurple
    assert parse_rgb("rgb(102,51,153)") == rebeccapurple


def test_partial_functions():
    # a few spot-checks
    assert black("test") == color("test", "black")
    assert red("test") == color("test", "red")
    assert green("test") == color("test", "green")

    assert bold("test") == color("test", style="bold")
    assert underline("test") == color("test", style="underline")


def test_doc_example():
    """Text examples given in documentation"""
    assert color("my string", fg="blue") == "\x1b[34mmy string\x1b[0m"
    assert (color("some text", fg="red", bg="yellow",
                  style="underline") == "\x1b[31;43;4msome text\x1b[0m")


def test_custom_partial():
    important = partial(color, fg="red", style="bold+underline")
    boldul = partial(color, style="bold+underline")

    text = "very important"
    answer = "\x1b[31;1;4mvery important\x1b[0m"

    assert color(text, fg="red", style="bold+underline") == answer
    assert red(text, style="bold+underline") == answer
    assert boldul(text, fg="red") == answer
    assert important(text) == answer


@pytest.mark.skipif(not _PY2, reason="only relevant for PY2")
def test_unicode_strings_py2():
    """
    Ensure that colors, given a str, returns a str. We already
    know from other tests that given a unicode, returns a unicode
    (else they would fail gloriously).
    """
    r1base = "This is good"  # note is str, not unicode
    r1 = red(r1base)
    assert r1 == "\x1b[31mThis is good\x1b[0m"
    assert strip_color(r1) == r1base
    assert isinstance(strip_color(r1), str)
    assert ansilen(r1) == len(r1base)
    assert isinstance(r1, str)


def test_unicode_strings():
    r1base = u"This is good"
    r1 = red(r1base)
    assert r1 == u"\x1b[31mThis is good\x1b[0m"
    assert strip_color(r1) == r1base
    assert ansilen(r1) == len(r1base)

    r2base = u"This\u2014is über good"
    r2 = red(r2base)
    assert r2 == u"\x1b[31mThis\u2014is \u00fcber good\x1b[0m"
    assert strip_color(r2) == r2base
    assert ansilen(r2) == len(r2base)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Tests for autosys package
"""
import importlib
import os
import sys
from pytest import raises
import pytest
# import autosys_system


# sys.path.insert(0, os.path.abspath("."))
# sys.path.insert(0, os.path.abspath("../"))
# syspy = importlib.import_module("sys-py")


def inc(x):
    return x + 1


def test_answer():
    assert inc(3) == 4


# _PY2 = sys.version_info[0] == 2



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" test_as_testing.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

# from __future__ import absolute_import, print_function
import pytest
from pytest import raises
from autosys.as_testing import *


def test_v_name():
    i = 25
    j = 3.14
    s = 'Newport News'
    t = 'fsadf$f%dj!d/fasd//'
    l = [1, 2, '3']
    d = dict(globals())
    assert v_name(i) == 'i'
    assert v_name(j) == 'j'
    assert v_name(s) == 's'
    assert v_name(t) == 't'
    assert v_name(l) == 'l'
    assert v_name(d) == 'd'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

from setuptools import find_namespace_packages, setup

from autosys import version

setup(
    name=version.name,
    version=version.__version__,
    description="System utilities for Python on macOS",
    long_description=open("README.md").read(),
    author="Michael Treanor",
    author_email="skeptycal@gmail.com",
    maintainer="Michael Treanor",
    maintainer_email="skeptycal@gmail.com",
    url="https://skeptycal.github.io/{}/".format(version.name),
    license="MIT",
    packages=find_namespace_packages(),
    install_requires=["tox", "pytest", "coverage", "pytest-cov"],
    test_suite="test",
    zip_safe=False,
    keywords="cli utilities python ai ml text console log debug test testing",
    package_data={
        # If any package contains txt, rst, md files, include them:
        "": ["*.txt", "*.rst", "*.md", "*.ini", "*.png", "*.jpg"]
    },
    project_urls={
        "Website": "https://skeptycal.github.io/{}/".format(version.name),
        # "Documentation": "https://docs.example.com/HelloWorld/",
        "Source Code": "https://www.github.com/skeptycal/{}/".format(version.name),
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: MacOS X",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Programming Language :: Cython",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
        "Topic :: Utilities",
    ],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
test_www.py
"""
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

from __future__ import absolute_import, print_function
import pytest
from pytest import raises
from autosys.as_www import url_test


def test_url_test():
    urls = ['http://www.google.com', 'https://www.google.com',
            'https://www.twitter.com/skeptycal', 'http://192.168.0.1']
    # 'https://www.skeptycal.com'
    for url in urls:
        assert url_test(url) == 200



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
as_ansi.py

The obligatory ANSI text colors implementation.
Requires Python 3.6+
"""
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal


from __future__ import absolute_import, print_function

if True:  # imports
    import decimal
    import fileinput
    import locale
    import math
    import os
    import random
    import sys
    import textwrap
    import time
    import timeit
    import logging
    import traceback
    from typing import Any, Dict, List


if True:
    from typing import Dict, List, Tuple, Any
    from autosys import __version__ as version
    from autosys import name
    import autosys.as_constants as const
    import autosys.as_system as syst


ANSI_CHART = """ANSI Effects Chart
    <code>
    ╔══════════╦════════════════════════════════╦═════════════════════════════════════════════════════════════════════════╗
    ║  Code    ║             Effect             ║                                   Note                                  ║
    ╠══════════╬════════════════════════════════╬═════════════════════════════════════════════════════════════════════════╣
    ║ 0        ║  Reset / Normal                ║  all attributes off                                                     ║
    ║ 1        ║  Bold or increased intensity   ║                                                                         ║
    ║ 2        ║  Faint (decreased intensity)   ║  NWS                                                  ║
    ║ 3        ║  Italic                        ║  NWS Sometimes treated as inverse.                    ║
    ║ 4        ║  Underline                     ║                                                                         ║
    ║ 5        ║  Slow Blink                    ║  less than 150 per minute                                               ║
    ║ 6        ║  Rapid Blink                   ║  MS-DOS ANSI.SYS; 150+ per minute; not widely supported                 ║
    ║ 7        ║  [[reverse video]]             ║  swap foreground and background colors                                  ║
    ║ 8        ║  Conceal                       ║  NWS                                                  ║
    ║ 9        ║  Crossed-out                   ║  Characters legible, but marked for deletion.  NWS    ║
    ║ 10       ║  Primary(default) font         ║                                                                         ║
    ║ 11–19    ║  Alternate font                ║  Select alternate font `n-10`                                           ║
    ║ 20       ║  Fraktur                       ║  hardly ever supported                                                  ║
    ║ 21       ║  Bold off or Double Underline  ║  Bold off not widely supported; double underline hardly ever supported. ║
    ║ 22       ║  Normal color or intensity     ║  Neither bold nor faint                                                 ║
    ║ 23       ║  Not italic, not Fraktur       ║                                                                         ║
    ║ 24       ║  Underline off                 ║  Not singly or doubly underlined                                        ║
    ║ 25       ║  Blink off                     ║                                                                         ║
    ║ 27       ║  Inverse off                   ║                                                                         ║
    ║ 28       ║  Reveal                        ║  conceal off                                                            ║
    ║ 29       ║  Not crossed out               ║                                                                         ║
    ║ 30–37    ║  Set foreground color          ║  See color table below                                                  ║
    ║ 38       ║  Set foreground color          ║  Next arguments are `5;n` or `2;r;g;b`, see below                       ║
    ║ 39       ║  Default foreground color      ║  implementation defined (according to standard)                         ║
    ║ 40–47    ║  Set background color          ║  See color table below                                                  ║
    ║ 48       ║  Set background color          ║  Next arguments are `5;n` or `2;r;g;b`, see below                       ║
    ║ 49       ║  Default background color      ║  implementation defined (according to standard)                         ║
    ║ 51       ║  Framed                        ║                                                                         ║
    ║ 52       ║  Encircled                     ║                                                                         ║
    ║ 53       ║  Overlined                     ║                                                                         ║
    ║ 54       ║  Not framed or encircled       ║                                                                         ║
    ║ 55       ║  Not overlined                 ║                                                                         ║
    ║ 60       ║  ideogram underline            ║  hardly ever supported                                                  ║
    ║ 61       ║  ideogram double underline     ║  hardly ever supported                                                  ║
    ║ 62       ║  ideogram overline             ║  hardly ever supported                                                  ║
    ║ 63       ║  ideogram double overline      ║  hardly ever supported                                                  ║
    ║ 64       ║  ideogram stress marking       ║  hardly ever supported                                                  ║
    ║ 65       ║  ideogram attributes off       ║  reset the effects of all of 60-64                                      ║
    ║ 90–97    ║  Set bright foreground color   ║  aixterm (not in standard)                                              ║
    ║ 100–107  ║  Set bright background color   ║  aixterm (not in standard)                                              ║
    ╚══════════╩════════════════════════════════╩═════════════════════════════════════════════════════════════════════════╝
    </code>
    """

#  38 - Set foreground color - Next arguments are `5;n` or `2;r;g;b`, see below
ANSI_DICT_LIST = [
    {'code': 'Code', 'effect': 'Effect', 'notes': 'Note'},
    {'code': '0', 'effect': 'Reset/Normal', 'notes': 'allattributesoff'},
    {'code': '1', 'effect': 'Boldorincreasedintensity', 'notes': ''},
    {'code': '2', 'effect': 'Faint(decreasedintensity)', 'notes': 'NWS'},
    {'code': '3', 'effect': 'Italic', 'notes': 'NWSSometimestreatedasinverse.'},
    {'code': '4', 'effect': 'Underline', 'notes': ''},
    {'code': '5', 'effect': 'SlowBlink', 'notes': 'lessthan150perminute'},
    {'code': '6', 'effect': 'RapidBlink',
        'notes': 'MS-DOSANSI.SYS;150+perminute;notwidelysupported'},
    {'code': '7', 'effect': '[[reversevideo]]',
        'notes': 'swapforegroundandbackgroundcolors'},
    {'code': '8', 'effect': 'Conceal', 'notes': 'NWS'},
    {'code': '9', 'effect': 'Crossed-out',
        'notes': 'Characterslegible,butmarkedfordeletion.NWS'},
    {'code': '10', 'effect': 'Primary(default)font', 'notes': ''},
    {'code': '11–19', 'effect': 'Alternatefont',
        'notes': 'Selectalternatefont`n-10`'},
    {'code': '20', 'effect': 'Fraktur', 'notes': 'hardlyeversupported'},
    {'code': '21', 'effect': 'BoldofforDoubleUnderline',
        'notes': 'Boldoffnotwidelysupported;doubleunderlinehardlyeversupported.'},
    {'code': '22', 'effect': 'Normalcolororintensity',
        'notes': 'Neitherboldnorfaint'},
    {'code': '23', 'effect': 'Notitalic,notFraktur', 'notes': ''},
    {'code': '24', 'effect': 'Underlineoff',
        'notes': 'Notsinglyordoublyunderlined'},
    {'code': '25', 'effect': 'Blinkoff', 'notes': ''},
    {'code': '27', 'effect': 'Inverseoff', 'notes': ''},
    {'code': '28', 'effect': 'Reveal', 'notes': 'concealoff'},
    {'code': '29', 'effect': 'Notcrossedout', 'notes': ''},
    {'code': '30', 'effect': 'Setforegroundcolor', 'notes': 'BLACK, add ;1m GRAY'},
    {'code': '31', 'effect': 'Setforegroundcolor',
        'notes': 'RED, add ;1m BRIGHTRED'},
    {'code': '32', 'effect': 'Setforegroundcolor',
        'notes': 'GREEN, add ;1m BRIGHTGREEN'},
    {'code': '33', 'effect': 'Setforegroundcolor',
        'notes': 'YELLOW, add ;1m BRIGHTYELLOW'},
    {'code': '34', 'effect': 'Setforegroundcolor',
        'notes': 'BLUE, add ;1m BRIGHTBLUE'},
    {'code': '35', 'effect': 'Setforegroundcolor',
        'notes': 'MAGENTA, add ;1m BRIGHTMAGENTA'},
    {'code': '36', 'effect': 'Setforegroundcolor',
        'notes': 'CYAN, add ;1m BRIGHTCYAN'},
    {'code': '37', 'effect': 'Setforegroundcolor',
        'notes': 'LIGHTGRAY, add ;1m WHITE'},
    # "BLACK": "\u001b[30m",
    # "RED": "\u001b[31m",
    # "GREEN": "\u001b[32m",
    # "BROWN": "\u001b[33m",
    # "BLUE": "\u001b[34m",
    # "MAGENTA": "\u001b[35m",
    # "CYAN": "\u001b[36m",
    # "GRAY": "\u001b[37m",
    # "BBLACK": "\u001b[30;1m",
    # "BRED": "\u001b[31;1m",
    # "BGREEN": "\u001b[32;1m",
    # "BYELLOW": "\u001b[33;1m",
    # "BBLUE": "\u001b[34;1m",
    # "BMAGENTA": "\u001b[35;1m",
    # "BCYAN": "\u001b[36;1m",
    # "BWHITE": "\u001b[37;1m",
    {'code': '38', 'effect': 'Setforegroundcolor',
        'notes': 'Nextargumentsare`5;n`or`2;r;g;b`,seebelow'},
    {'code': '39', 'effect': 'Defaultforegroundcolor',
        'notes': 'implementationdefined(accordingtostandard)'},
    {'code': '40–47', 'effect': 'Setbackgroundcolor', 'notes': 'Seecolortablebelow'},
    {'code': '48', 'effect': 'Setbackgroundcolor',
        'notes': 'Nextargumentsare`5;n`or`2;r;g;b`,seebelow'},
    {'code': '49', 'effect': 'Defaultbackgroundcolor',
        'notes': 'implementationdefined(accordingtostandard)'},
    {'code': '51', 'effect': 'Framed', 'notes': ''},
    {'code': '52', 'effect': 'Encircled', 'notes': ''},
    {'code': '53', 'effect': 'Overlined', 'notes': ''},
    {'code': '54', 'effect': 'Notframedorencircled', 'notes': ''},
    {'code': '55', 'effect': 'Notoverlined', 'notes': ''},
    {'code': '60', 'effect': 'ideogramunderline', 'notes': 'hardlyeversupported'},
    {'code': '61', 'effect': 'ideogramdoubleunderline',
        'notes': 'hardlyeversupported'},
    {'code': '62', 'effect': 'ideogramoverline', 'notes': 'hardlyeversupported'},
    {'code': '63', 'effect': 'ideogramdoubleoverline',
        'notes': 'hardlyeversupported'},
    {'code': '64', 'effect': 'ideogramstressmarking', 'notes': 'hardlyeversupported'},
    {'code': '65', 'effect': 'ideogramattributesoff',
        'notes': 'resettheeffectsofallof60-64'},
    {'code': '90–97', 'effect': 'Setbrightforegroundcolor',
        'notes': 'aixterm(notinstandard)'},
    {'code': '100–107', 'effect': 'Setbrightbackgroundcolor', 'notes': 'aixterm(notinstandard)'}]


FG_DICT: Dict[str, str] = {
    # The first 11 codes are just favorites I picked.
    # Some have short and long names for duplicates.
    # Add more personalized colors by name if desired ...
    "MAIN": "\u001b[38;5;229m",
    "COOL": "\u001b[38;5;38m",
    "MIKES_BLUE": "\u001b[38;5;38m",
    "GO": "\u001b[38;5;28m",
    "GREEN_MEANS_GO": "\u001b[38;5;28m",
    "WARN": "\u001b[38;5;203m",
    "CHERRY": "\u001b[38;5;124m",
    "CANARY": "\u001b[38;5;226m",
    "DANGERWILLROBINSON": "\u001b[38;5;203m",
    "ATTN": "\u001b[38;5;178m",
    "WHATABURGER": "\u001b[38;5;178m",
    "PURPLE": "\u001b[38;5;93m",
    "PURPLEHAZE": "\u001b[38;5;93m",
    "RESET": "\u001b[0m",  # Reset all attributes
    "NEXT_FG": "\u001b[38m",  # Next Foreground in Cycle (rainbow ...)
    "DEF_FG": "\u001b[39m",  # Reset to Default Foreground
    # These are the codes for the foreground 8 color set
    "BLACK": "\u001b[30m",      # 0
    "RED": "\u001b[31m",        # 1
    "GREEN": "\u001b[32m",      # 2
    "BROWN": "\u001b[33m",      # 3
    "BLUE": "\u001b[34m",       # 4
    "MAGENTA": "\u001b[35m",    # 5
    "CYAN": "\u001b[36m",       # 6
    "GRAY": "\u001b[37m",       # 7
    # These are added to the 8-color FG to make the 16 color FG set
    "BBLACK": "\u001b[30;1m",
    "BRED": "\u001b[31;1m",
    "BGREEN": "\u001b[32;1m",
    "BYELLOW": "\u001b[33;1m",
    "BBLUE": "\u001b[34;1m",
    "BMAGENTA": "\u001b[35;1m",
    "BCYAN": "\u001b[36;1m",
    "BWHITE": "\u001b[37;1m",
}

BG_DICT: Dict[str, str] = {
    # The first 11 codes are just favorites I picked.
    # Some have short and long names for duplicates.
    # Add more personalized colors by name if desired ...
    "MAIN": "\u001b[48;5;185m",
    "COOL": "\u001b[48;5;38m",
    "MIKES_BLUE": "\u001b[48;5;38m",
    "GO": "\u001b[48;5;28m",
    "GREEN_MEANS_GO": "\u001b[48;5;28m",
    "WARN": "\u001b[48;5;203m",
    "CHERRY": "\u001b[48;5;124m",
    "CANARY": "\u001b[48;5;226m",
    "DANGERWILLROBINSON": "\u001b[48;5;203m",
    "ATTN": "\u001b[48;5;178m",
    "WHATABURGER": "\u001b[48;5;178m",
    "PURPLE": "\u001b[48;5;93m",
    "PURPLEHAZE": "\u001b[48;5;93m",
    "RESET": "\u001b[0m",  # Reset all attributes
    "NEXT_BG": "\u001b[48m",  # Next Background in Cycle (rainbow ...)
    "DEF_BG": "\u001b[49m",  # Reset to Default Background
    # These are the codes for the background 8 color set
    "BG_BLACK": "\u001b[40m",
    "BG_RED": "\u001b[41m",
    "BG_GREEN": "\u001b[42m",
    "BG_BROWN": "\u001b[43m",
    "BG_BLUE": "\u001b[44m",
    "BG_MAGENTA": "\u001b[45m",
    "BG_CYAN": "\u001b[46m",
    "BG_GRAY": "\u001b[47m",
    # These are added to the 8-color BG to make the 16 color BG set
    "BG_BBlack": "\u001b[40;1m",
    "BG_BRed": "\u001b[41;1m",
    "BG_BGreen": "\u001b[42;1m",
    "BG_BYellow": "\u001b[43;1m",
    "BG_BBlue": "\u001b[44;1m",
    "BG_BMagenta": "\u001b[45;1m",
    "BG_BCyan": "\u001b[46;1m",
    "BG_BWhite": "\u001b[47;1m",
}

FLAGS_DICT: Dict[str, str] = {
    # ANSI STANDARD ATTRIBUTES
    "NONE": "",
    "RESET": "\u001b[0m",              # Reset / Normal
    "BOLD": "\u001b[1m",               # Bold
    "DIM": "\u001b[2m",                # Dim
    "ITALIC": "\u001b[3m",             # Italic
    "UNDER": "\u001b[4m",              # Underline
    "REVERSE": "\u001b[7m"
    # "CONCEAL": '\u001b[8m',          # Conceal               NWS
    # "STRIKE": '\u001b[9m',           # Strike-through        Crossed-out
}


def _join(*values):
    """
    (ANSI encoding) Join a series of values with semicolons. The values
    are either integers or strings.
    """
    return ";".join(str(v) for v in values)


def print_ansi_list():
    """ Print list of ansi color codes """
    for line in ANSI_DICT_LIST:
        print(line)


# Fills in the 256 set color codes
#   => (0 to 231) colors in the format 'COLOR133'
for _ in range(0, 232):
    FG_DICT["COLOR" + str(_)
            ] = "\u001b[38;5;" + str(_) + "m"
    BG_DICT["COLOR" + str(_)
            ] = "\u001b[48;5;" + str(_) + "m"

#   => (232 to 255) grayscale in the format 'GRAY243'
for _ in range(232, 255):
    key = "GRAY" + str(_)
    FG_DICT["GRAY" + str(_)] = "\u001b[38;5;" + \
        str(_) + "m"
    BG_DICT["GRAY" + str(_)] = "\u001b[48;5;" + \
        str(_) + "m"

# Turn off color when using ipython, otherwise leave it on
ENABLE_COL1111111111111111111111111111111111111111111111111

DEFAULT_FG_COLOR: str = "MAIN"
DEFAULT_BG_COLOR: str = "BG_BLACK"
DEFAULT_FLAGS: str = "RESET"


def color_encode(
        fg_color: str = DEFAULT_FG_COLOR,
        bg_color: str = DEFAULT_BG_COLOR,
        flags_color: str = DEFAULT_FLAGS,
) -> str:
    """ Returns encoded ANSI text string from components.

        Keyword Arguments:
                fg_color {str} - ANSI foreground color {DEFAULT_FG_COLOR}
                bg_color {str} - ANSI background color {DEFAULT_BG_COLOR}
                flags_color {str} - ANSI format codes

        Returns:
            str - printable escaped ANSI text formatting string
        """
    s: str = ''
    if ENABLE_COLOR:
        s += FG_DICT[fg_color]
        s += BG_DICT[bg_color]
        s += FLAGS_DICT[flags_color]
    return s


DEFAULT_ENCODE: str = color_encode(
    DEFAULT_FG_COLOR, DEFAULT_BG_COLOR, DEFAULT_FLAGS)
STICKY_ENCODE: str = DEFAULT_ENCODE
RESET_ENCODE: str = color_encode("DEF_FG", "DEF_BG", "RESET")


def color_print(color_code: str, *args: Any, **kwargs: Any) -> int:
    """ Print using ANSI color codes

        Arguments:
            color_code {str} - printable escaped ANSI text formatting string
            args {tuple} - passthrough for print arguments
            kwargs {dict} - passthrough for print options

        Return:
            int: 0 for success; > 0 for error code """
    try:
        if ENABLE_COLOR:
            print(color_code, *args, STICKY_ENCODE, **kwargs)
        else:
            print(*args, **kwargs)
    except RuntimeError:
        return 1
    return 0


def color_gradient(color_1: int, color_2: int, print_string: str) -> int:
    """ Print <print_string> using a calculated ANSI color gradient
            that changes evenly and gradually between color_1 and color_2.

        Arguments:
            color_1 {int} - ANSI color code(256 set) - lower gradient bound
            color_2 {int} - ANSI color code(256 set) - upper gradient bound
            print_string {str} - target string

        Returns:
            int: 0 for success; > 0 for error code """
    if ENABLE_COLOR and len(print_string) >= 4:
        result: str = ''
        # make sure color_2 is the larger number
        if color_1 > color_2:
            color_1, color_2 = color_2, color_1
        try:
            gradient_slope: float = (color_2 - color_1) / len(print_string)
            for i, _ in enumerate(print_string):
                color_code = "COLOR" + str(int(color_1 + i * gradient_slope))
                result += color_encode(color_code) + print_string[i]
            print(result)
        except TypeError:
            return 1
    return 0


def color_cycle(color_1: int, print_string: str) -> int:
    """ Print by cycling through colors starting at <color_1>.

        Arguments:
            color_1 {int} - ANSI color code(256 set) - lower gradient bound
            print_string {str} - target string

        Returns:
            int: 0 for success; > 0 for error code """
    if ENABLE_COLOR:
        try:
            for _, c in enumerate(print_string):
                color_1 += 1
                if color_1 < 1 or color_1 > 230:
                    color_1 = 1
                color_code = "COLOR" + str(color_1)
                print(color_encode(color_code), str(c), sep="", end="")
            print(DEFAULT_ENCODE, ' ', sep="", end="")
        except (RuntimeError, IndexError):
            return 1
    return 0


def loading(count: int) -> int:
    """ Demo - Print <count> text progress bars.
            They are non responsive and synchronous.

        Arguments:
            count {int} - number of bars

        Returns:
            int: 0 for success; > 0 for error code """

    width: int = 0
    w_string: str = ""
    all_progress = [0] * count
    sys.stdout.write("\n" * count)  # space to draw the bars
    while any(x < 100 for x in all_progress):
        time.sleep(0.003)
        # Randomly increment one of our progress values
        unfinished = [(i, v) for (i, v) in enumerate(all_progress) if v < 100]
        index, _ = random.choice(unfinished)
        all_progress[index] += 1
        # Draw the progress bars
        sys.stdout.write(u"\u001b[1000D")  # Move left
        sys.stdout.write(u"\u001b[" + str(count) + "A")  # Move up
        for progress in all_progress:
            width = int(progress / 4)
            w_string = "#" * width
            pad_string = " " * (25 - width)
            if len(w_string) >= 5:
                pad_string = " " * (25 - width)
                pad_string = "|" + w_string + pad_string + "]"
            else:
                pad_string = " " * 21
                pad_string = "|" + w_string + pad_string + "]"
            color_cycle(124, pad_string)
    return 0


def fg_samples():
    """ Foreground color samples """
    i: int = 0
    for k in FG_DICT:
        if i < 32:
            if not i % 8:
                print()
        elif not i % 6:
            print()
        i += 1
        color_print(
            color_encode(k, DEFAULT_BG_COLOR),
            k.replace("OLOR", ""),
            " ",
            sep="",
            end="",
        )
    print()


def bg_samples():
    """ Background color samples """
    i = 0
    for k in BG_DICT:
        if i < 32:
            if not i % 8:
                print()
        elif not i % 6:
            print()
        i += 1
        color_print(
            color_encode(DEFAULT_FG_COLOR, k),
            k.replace("OLOR", ""),
            " ",
            sep="",
            end="",
        )
    print()


def flags_samples():
    """ ANSI Flags samples """
    print()
    i: int = 0
    s: str = 'Now is the time for all good men' + \
        'to come to the aid of their country.'
    for k in FLAGS_DICT:
        i += 1
        color_print(
            color_encode("BWHITE", "PURPLEHAZE", k),
            i,
            " : ",
            str(k),
            "  \t",
            s,
            RESET_ENCODE,
            sep="",
        )
    print()
    color_gradient(
        130,
        144,
        "Gradient!!!  \t{}\n".format(s),
    )
    print()
    color_cycle(
        130,
        "Cycle!!!  \t{}\n".format(s*5)
    )
    print()
    # if ENABLE_COLOR:
    #     loading(5)
    # print()
    color_print(DEFAULT_ENCODE)
    color_print(color_encode("BWHITE", "PURPLEHAZE",
                             "ITALIC"), "TESTS COMPLETE ...")


if __name__ == "__main__":
    # TEST SAMPLES to use if script is run from the command lines
    USAGE = (
        version + """

author    - Michael Treanor  <skeptycal@gmail.com>
copyright - 2019 (c) Michael Treanor
license   - MIT <https://opensource.org/licenses/MIT>
github    - https://www.github.com/skeptycal

Usage: text_colors {demo|version|help}

  Parameters:
      demo, -d, --demo        -- install and initialize
      version, -v, --version  -- display version information
      help, -h, --help        -- display usage and information"""
    )

    if len(sys.argv) < 2:
        # color_cycle(42, usage)
        color_gradient(42, 222, USAGE)
    else:
        arg1: str = sys.argv[1].lower()
        if arg1 in ["version", "-v", "--version"]:
            color_cycle(88, version)
        elif arg1 in ["help", "-h", "--help"]:
            color_print(color_encode(
                "BWHITE", "PURPLEHAZE", "ITALIC"), version)
            print()
            color_cycle(88, ANSI_CHART)
        elif arg1 in ["demo", "-d", "--demo"]:
            fg_samples()
            bg_samples()
            flags_samples()
        else:
            color_gradient(42, 222, USAGE)
            # color_cycle(42, usage)

    print("\n\n")
    print("python version (pyver): ", syst.pyver())
    print("python shell (py_shell): ", syst.py_shell())

# Resources:

# https://www.geeksforgeeks.org/print-colors-python-terminal/

# https://stackoverflow.com/questions/287871/print-in-terminal-with-colors
#   1,000,002 ways to do this ... but I still did it my own way ...

# https://en.wikipedia.org/wiki/ANSI_escape_code

# Sequences have different lengths.
#   All sequences start with ESC (27 / hex 0x1B / octal 033), followed
#   by a second byte in the range 0x40 - 0x5F (ASCII @A-Z[\\]^_)



import re
import sys
from functools import partial

from .csscolors import css_colors, parse_rgb

# previous support for Python 2.x string types
# _PY2 = sys.version_info[0] == 2
# string_types = basestring if _PY2 else str
string_types = str

template = "\x1b[{0}m{1}\x1b[0m"


# ANSI color names. There is also a "default"
COLORS = ("black", "red", "green", "yellow", "blue", "magenta", "cyan", "white")

# ANSI style names
STYLES = (
    "none",
    "bold",
    "faint",
    "italic",
    "underline",
    "blink",
    "blink2",
    "negative",
    "concealed",
    "crossed",
)


def is_string(obj):
    """
    Is the given object a string?
    """
    return isinstance(obj, string_types)


def _join(*values):
    """
    Join a series of values with semicolons. The values
    are either integers or strings, so stringify each for
    good measure. Worth breaking out as its own function
    because semicolon-joined lists are core to ANSI coding.
    """
    return ";".join(str(v) for v in values)


def _color_code(spec, base):
    """
    Workhorse of encoding a color. Give preference to named colors from
    ANSI, then to specific numeric or tuple specs. If those don't work,
    try looking up CSS color names or parsing CSS color specifications
    (hex or rgb).

    :param str|int|tuple|list spec: Unparsed color specification
    :param int base: Either 30 or 40, signifying the base value
        for color encoding (foreground and background respectively).
        Low values are added directly to the base. Higher values use `
        base + 8` (i.e. 38 or 48) then extended codes.
    :returns: Discovered ANSI color encoding.
    :rtype: str
    :raises: ValueError if cannot parse the color spec.
    """
    if is_string(spec):
        spec = spec.strip().lower()

    if spec == "default":
        return _join(base + 9)
    elif spec in COLORS:
        return _join(base + COLORS.index(spec))
    elif isinstance(spec, int) and 0 <= spec <= 255:
        return _join(base + 8, 5, spec)
    elif isinstance(spec, (tuple, list)):
        return _join(base + 8, 2, _join(*spec))
    else:
        rgb = parse_rgb(spec)
        # parse_rgb raises ValueError if cannot parse spec
        # or returns an rgb tuple if it can
        return _join(base + 8, 2, _join(*rgb))


def color(s, fg=None, bg=None, style=None):
    """
    Add ANSI colors and styles to a string.

    :param str s: String to format.
    :param str|int|tuple fg: Foreground color specification.
    :param str|int|tuple bg: Background color specification.
    :param str: Style names, separated by '+'
    :returns: Formatted string.
    :rtype: str (or unicode in Python 2, if s is unicode)
    """
    codes = []

    if fg:
        codes.append(_color_code(fg, 30))
    if bg:
        codes.append(_color_code(bg, 40))
    if style:
        for style_part in style.split("+"):
            if style_part in STYLES:
                codes.append(STYLES.index(style_part))
            else:
                raise ValueError('Invalid style "%s"' % style_part)

    if codes:
        template = "\x1b[{0}m{1}\x1b[0m"
        if _PY2 and isinstance(s, unicode):
            # Take care in PY2 to return str if str is given, or unicode if
            # unicode given. A pain, but PY2's fragility with Unicode makes it
            # important to avoid disruptions (including gratuitous up-casting
            # of str to unicode) that might trigger downstream errors.
            template = unicode(template)
        return template.format(_join(*codes), s)
    else:
        return s


def strip_color(s):
    """
    Remove ANSI color/style sequences from a string. The set of all possible
    ANSI sequences is large, so does not try to strip every possible one. But
    does strip some outliers seen not just in text generated by this module, but
    by other ANSI colorizers in the wild. Those include `\x1b[K` (aka EL or
    erase to end of line) and `\x1b[m`, a terse version of the more common
    `\x1b[0m`.
    """
    return re.sub("\x1b\\[(K|.*?m)", "", s)


def ansilen(s):
    """
    Given a string with embedded ANSI codes, what would its
    length be without those codes?
    """
    return len(strip_color(s))


# Foreground color shortcuts
black = partial(color, fg="black")
red = partial(color, fg="red")
green = partial(color, fg="green")
yellow = partial(color, fg="yellow")
blue = partial(color, fg="blue")
magenta = partial(color, fg="magenta")
cyan = partial(color, fg="cyan")
white = partial(color, fg="white")

# Style shortcuts
bold = partial(color, style="bold")
none = partial(color, style="none")
faint = partial(color, style="faint")
italic = partial(color, style="italic")
underline = partial(color, style="underline")
blink = partial(color, style="blink")
blink2 = partial(color, style="blink2")
negative = partial(color, style="negative")
concealed = partial(color, style="concealed")
crossed = partial(color, style="crossed")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_80col.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

from autosys import *
from autosys.as_system import py_shell
from autosys.as_constants import *
from autosys.as_constants import PY_ENV as PY_ENV

# make sure we're testing colors module in this dir, not system
from autosys.colors import COLORS, STYLES, color

sys.path.insert(0, os.path.abspath("."))

# Turn off color when using ipython, otherwise leave it on
ENABLE_COLOR: bool = True
if py_shell() in ["ipython", "ipython-notebook"]:
    ENABLE_COLOR = False


def test_styles(bg, fg):
    for style in (None, ) + STYLES:
        cname = fg or "default"
        # renamed None color to default to avoid confusion wiht normal style
        if cname.startswith("bright"):
            cname = cname[6:].upper()
        text = cname[:5].ljust(6)
        print(color(text, fg=fg, bg=bg, style=style), end=" ")
    print()


# doubled number of colors, so have to split test into halves to
# test on standard 80-column terminal

colors = [None] + list(COLORS[:8])
brights = list(COLORS[8:])

for bg in colors:
    for fg in colors:
        test_styles(bg, fg)
    for fg in brights:
        test_styles(bg, fg)
for bg in brights:
    for fg in colors:
        test_styles(bg, fg)
    for fg in brights:
        test_styles(bg, fg)

for i in range(256):
    if i % 64 == 0:
        print()
    print(color(" ", bg=i), end="")

print()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_system.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

# from __future__ import absolute_import, print_function
# from typing import Any, Dict, FrozenSet, List, Sequence, Tuple
from autosys import *


def _get_builtins():
    return sys.builtin_module_names


def basename(filename: str) -> str:
    """
    get only basename from full path
    # TODO translate to pathlib
    """
    return os.path.basename(filename)


def py_path() -> List[str]:
    """ Return list of current python path elements. """
    try:
        return os.environ['PYTHONPATH'].split(os.pathsep)
    except KeyError:
        return []


def py3up() -> bool:
    """ Return True if 'Python >= 3' else False

        If you want to detect pre-Python 3 and don't want to import anything...
        ... you can (ab)use list comprehension scoping changes
    """
    # https://stackoverflow.com/questions/1093322/how-do-i-check-what-version-of-python-is-running-my-script/35294211
    # https://stackoverflow.com/a/52825819/9878098
    return (lambda x: [x for x in [False]] and None or x)(True)


def pyver() -> str:
    """ Returns string with python version number in major.minor.micro format.
            (e.g. 3.7.3  or  2.7.12)
    """
    return '.'.join(str(i) for i in __import__('sys').version_info[:3])


def py_shell() -> str:
    """ Returns string containing current python shell name. """

    shell: str = ""
    # PY_ENV = os.environ
    # PY_BASE = os.path.basename(PY_ENV["_"])
    if "JPY_PARENT_PID" in PY_ENV:
        shell = "ipython notebook"
    elif "pypy" in PY_ENV:
        shell = "pypy"
    elif "jupyter-notebook" in PY_BASE:
        shell = "jupyter notebook"
    elif "ipython" in PY_BASE:
        shell = "ipython"
    else:
        try:
            import platform

            shell = platform.python_implementation()
        except ImportError:
            pass
    # print("pyshell() output: ", shell.strip())
    return shell.strip()


def _pprint_globals():
    """
    Pretty Print all global variables.
    Designed for debugging purposes.
    """
    print()
    print("Globals: ")
    print("*" * 40)
    width = max(len(i) for i in globals())
    print(f'(key width: {width})')
    for s in globals():
        print("{:<15.15} : {:<64.64}".format(s, str(globals().get(s))))
    print()


def njoin(l: List[str]) -> str:
    return '\n'.join(l)


def _pprint_code_tests(tests: List[Exception]):
    pass


def _get_functions():
    pass


def _pprint_dict_table(data: Dict[Any, Any] = globals(),
                       title: str = "",
                       borders: str = 'text'):
    """
    Pretty Print dictionary in table format.
    parameters:
        data: Dict[Any, Any] - dictionary containing table data
        title: str - Displayed Title of Data Table (Title Case Used)
        borders: str - border format (single, double, graphic, text*)
    """
    if title == "":
        title = data.__repr__()
    print(data.__repr__())
    print(data.__str__())
    maxwidth: int = 75  # 80 - 1 for each outer border and 3 for center padding
    width: int = max(len(i) for i in data) + 1
    width2: int = maxwidth - width
    if SET_DEBUG:
        print()
        print("Title: ", title.title())
        print("*" * len(title))
        print("width: ", width)
        print("width2: ", width2)

    fmt = ("{:<" + str(width) + "." + str(width) + "} : " + "{:<" +
           str(width2) + "." + str(width2) + "}")

    print("*" * 80)
    for s in globals():
        print(fmt.format(s, str(globals().get(s))))
    print("*" * 80)


def _execute_test_code(tests: List[str]) -> List[Exception]:
    result: List[Exception] = []
    for test in tests:
        try:
            # run test code
            print("=> ", test)
            exec(test)
        except Exception as e:
            result.append(e)
    return result


def _run_tests(tests: List[str] = []) -> str:
    if tests == []:
        tests: List[str] = [
            "_pprint_globals()",
            'print("basename: ", basename(__file__))',
            "print(1/0)",
        ]

    results = _execute_test_code(tests)
    for e in results:
        print("e.__class__.__name__", e.__class__.__name__)
        # log.exception(e)
        print("e: ", e)
        print("e.args: ", e.args)
        print("type(e): ", type(e))


if __name__ == "__main__":
    # log = logging.getLogger()
    print(_run_tests())
    print()
    print("... Tests Complete.")
    _pprint_dict_table

    print(_get_builtins)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_www.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal
from __future__ import absolute_import, print_function

import sys
import os
import requests


CONNECT_ERRORS = (ConnectionError, ConnectionAbortedError,
                  ConnectionResetError, ConnectionRefusedError)


def url_test(url: str = "http://www.google.com") -> int:
    """
    Return response from server request.
    """
    try:
        r = requests.get(url)
    except CONNECT_ERRORS as e:
        print(f'Connection error: {e}')
        return 0
    return r.status_code


if __name__ == "__main__":

    print()
    urls = ['http://www.google.com', 'https://www.google.com',
            'https://www.twitter.com/skeptycal']
    # 'https://www.skeptycal.com'
    for url in urls:
        res = url_test(url)
        print()
        if res == 200:
            print(f'Successful connection to {url} ...response: {res}')
        else:
            print(f'No connection to {url} ... response: {res}')
